@@ -73,7 +73,7 @@
     }
     self.osoFetcher.subtitleLanguageId = selectedLocale;
     self.activityIndicatorView = [[UIActivityIndicatorView alloc] init];
-    if (@available(iOS 13.0, *)) {
+    if (@available(iOS 13.0, tvOS 13.0, *)) {
         self.activityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleLarge;
     } else {
         self.activityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhiteLarge;
@@ -269,9 +269,10 @@
 
 - (void)VLCOSOFetcher:(VLCOSOFetcher *)aFetcher subtitleDownloadSucceededForItem:(VLCSubtitleItem *)subtitleItem atPath:(NSString *)pathToFile
 {
+    APLog(@"%s: %@", __func__, subtitleItem.name);
     [self stopActivity];
     VLCPlaybackService *vpc = [VLCPlaybackService sharedInstance];
-    [vpc openVideoSubTitlesFromFile:pathToFile];
+    [vpc addSubtitlesToCurrentPlaybackFromURL:[NSURL fileURLWithPath:pathToFile]];
     [self dismissViewControllerAnimated:YES completion:nil];
     [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackMetadataDidChange object:nil];
 }
@@ -389,18 +390,47 @@
 
         VLCSubtitleItem *item = self.searchResults[indexPath.row];
         NSString *subStorageLocation;
+        NSFileManager *fileManager = [NSFileManager defaultManager];
 #if TARGET_OS_IOS
         /* on iOS, we try to retain the subtitles if the played media is stored locally */
         VLCPlaybackService *vpc = [VLCPlaybackService sharedInstance];
         NSURL *mediaURL = vpc.currentlyPlayingMedia.url;
         if (mediaURL.isFileURL) {
-            subStorageLocation = [[mediaURL.path stringByDeletingPathExtension] stringByAppendingPathExtension:item.format];
-        } else {
+            /* that the media is a file URL does not mean that we can dump a subtitles file next to it
+             * tl;dr we may write to the Documents-folder only, but not to the potential Inbox folder stored within
+             * that needs to be treated as read-only */
+            NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
+            NSString *documentFolderPath = [searchPaths firstObject];
+            NSString *potentialInboxFolderPath = [documentFolderPath stringByAppendingPathComponent:@"Inbox"];
+            NSString *mediaURLpath = mediaURL.path;
+            if (![mediaURLpath containsString:potentialInboxFolderPath] && [mediaURLpath containsString:documentFolderPath]) {
+                /* the media is stored in the Documents folder but not in Inbox */
+                NSString *mediaPath = mediaURL.path;
+                subStorageLocation = [[mediaPath stringByDeletingPathExtension] stringByAppendingPathExtension:item.format];
+
+                /* make extra sure that we may write in the surrounding folder, otherwise drop it and save in caches */
+                if (![fileManager isWritableFileAtPath:[mediaPath stringByDeletingLastPathComponent]]) {
+                    subStorageLocation = nil;
+                }
+            }
+
+            if (!subStorageLocation) {
+                /* cache the downloaded subtitle in a writeable cache folder that is eventually emptied by the OS */
+                NSString *mediaFileName = mediaURL.path.lastPathComponent.stringByDeletingPathExtension;
+                searchPaths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
+                NSString *cacheFolderPath = [searchPaths.firstObject stringByAppendingPathComponent:kVLCSubtitlesCacheFolderName];
+                [fileManager createDirectoryAtPath:cacheFolderPath withIntermediateDirectories:YES attributes:nil error:nil];
+                subStorageLocation = [cacheFolderPath stringByAppendingFormat:@"/%@.%@", mediaFileName, item.format];
+            }
+        }
+        if (!subStorageLocation) {
 #endif
-        NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
-        NSString *folderPath = [searchPaths[0] stringByAppendingPathComponent:@"tempsubs"];
-        [[NSFileManager defaultManager] createDirectoryAtPath:folderPath withIntermediateDirectories:YES attributes:nil error:nil];
-        subStorageLocation = [folderPath stringByAppendingPathComponent:item.name];
+            /* media is not a file or we are on tvOS, then just store the downloaded subtitle under its own name
+             * and have it deleted by the OS some day through the cache folder */
+            NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
+            NSString *folderPath = [searchPaths.firstObject stringByAppendingPathComponent:kVLCSubtitlesCacheFolderName];
+            [fileManager createDirectoryAtPath:folderPath withIntermediateDirectories:YES attributes:nil error:nil];
+            subStorageLocation = [folderPath stringByAppendingPathComponent:item.name];
 #if TARGET_OS_IOS
         }
 #endif
