@@ -104,7 +104,6 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if (self) {
         _fullscreenSessionRequested = YES;
         // listen to audiosessions and appkit callback
-        _externalAudioPlaybackDeviceConnected = [self isExternalAudioPlaybackDeviceConnected];
         NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
         [defaultCenter addObserver:self selector:@selector(audioSessionRouteChange:)
                               name:AVAudioSessionRouteChangeNotification object:nil];
@@ -140,6 +139,9 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
         _mediaList = [[VLCMediaList alloc] init];
 
         _openedLocalURLs = [[NSMutableArray alloc] init];
+        dispatch_async(dispatch_get_main_queue(), ^{
+            self->_externalAudioPlaybackDeviceConnected = [self isExternalAudioPlaybackDeviceConnected];
+        });
     }
     return self;
 }
@@ -154,9 +156,9 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 }
 #pragma mark - playback management
 
-- (void)openVideoSubTitlesFromFile:(NSString *)pathToFile
+- (void)addSubtitlesToCurrentPlaybackFromURL:(NSURL *)subtitleURL
 {
-    [_mediaPlayer addPlaybackSlave:[NSURL fileURLWithPath:pathToFile] type:VLCMediaPlaybackSlaveTypeSubtitle enforce:YES];
+    [_mediaPlayer addPlaybackSlave:subtitleURL type:VLCMediaPlaybackSlaveTypeSubtitle enforce:YES];
 }
 
 - (void)playMediaList:(VLCMediaList *)mediaList firstIndex:(NSInteger)index subtitlesFilePath:(NSString * _Nullable)subsFilePath
@@ -219,12 +221,13 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     _actualVideoOutputView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
     _actualVideoOutputView.autoresizesSubviews = YES;
 
-    /* the chromecast options cannot be set per media, so we need to set it per
+    /* the chromecast and audio options cannot be set per media, so we need to set it per
      * media player instance however, potentially initialising an additional library instance
      * for this is costly, so this should be done only if needed */
     NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
     BOOL chromecastPassthrough = [[userDefaults objectForKey:kVLCSettingCastingAudioPassthrough] boolValue];
     int chromecastQuality = [[userDefaults objectForKey:kVLCSettingCastingConversionQuality] intValue];
+    BOOL audioTimeStretch = [[userDefaults objectForKey:kVLCSettingStretchAudio] boolValue];
     NSMutableArray *libVLCOptions = [NSMutableArray array];
     if (chromecastPassthrough) {
         [libVLCOptions addObject:[@"--" stringByAppendingString:kVLCSettingCastingAudioPassthrough]];
@@ -232,6 +235,9 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if (chromecastQuality != 2) {
         [libVLCOptions addObject:[NSString stringWithFormat:@"--%@=%i", kVLCSettingCastingConversionQuality, chromecastQuality]];
     }
+    if (!audioTimeStretch) {
+        [libVLCOptions addObject:[NSString stringWithFormat:@"--no-%@", kVLCSettingStretchAudio]];
+    }
     if (libVLCOptions.count > 0) {
         _listPlayer = [[VLCMediaListPlayer alloc] initWithOptions:libVLCOptions
                                                       andDrawable:_actualVideoOutputView];
@@ -240,13 +246,20 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     }
     _listPlayer.delegate = self;
 
+    NSMutableArray *debugLoggers = [NSMutableArray array];
 #if MEDIA_PLAYBACK_DEBUG
-    _listPlayer.mediaPlayer.libraryInstance.debugLogging = YES;
-    _listPlayer.mediaPlayer.libraryInstance.debugLoggingLevel = 4;
+    VLCConsoleLogger *consoleLogger = [[VLCConsoleLogger alloc] init];
+    consoleLogger.level = kVLCLogLevelDebug;
+    [debugLoggers addObject:consoleLogger];
 #endif
     BOOL saveDebugLogs = [userDefaults boolForKey:kVLCSaveDebugLogs];
     if (saveDebugLogs) {
-        NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
+        NSArray *searchPaths;
+#if TARGET_OS_IOS
+        searchPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
+#else
+        searchPaths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
+#endif
         NSString* logFilePath = [searchPaths[0] stringByAppendingPathComponent:@"Logs"];
         NSFileManager *fileManager = [NSFileManager defaultManager];
         ret = [fileManager fileExistsAtPath:logFilePath];
@@ -259,8 +272,15 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
         [dateFormatter setDateFormat:@"yyyy-MM-dd--HH-mm-ss"];
         logFilePath = [logFilePath stringByAppendingPathComponent:[NSString stringWithFormat: @"vlcdebug-%@.log", [dateFormatter stringFromDate:date]]];
         APLog(@"logging at '%@'", logFilePath);
-        [_listPlayer.mediaPlayer.libraryInstance setDebugLoggingToFile:logFilePath];
+        [fileManager createFileAtPath:logFilePath contents:nil attributes:nil];
+        NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath:logFilePath];
+        if (fileHandle) {
+            VLCFileLogger *fileLogger = [[VLCFileLogger alloc] initWithFileHandle:fileHandle];
+            fileLogger.level = kVLCLogLevelDebug;
+            [debugLoggers addObject:fileLogger];
+        }
     }
+    [_listPlayer.mediaPlayer.libraryInstance setLoggers:debugLoggers];
 
     id<VLCFilter> newFilter = _listPlayer.mediaPlayer.adjustFilter;
     [newFilter applyParametersFrom:_adjustFilter.mediaPlayerAdjustFilter];
@@ -269,8 +289,9 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     _mediaPlayer = _listPlayer.mediaPlayer;
 
     [_mediaPlayer setDelegate:self];
-    if ([[defaults objectForKey:kVLCSettingPlaybackSpeedDefaultValue] floatValue] != 0)
-        [_mediaPlayer setRate: [[defaults objectForKey:kVLCSettingPlaybackSpeedDefaultValue] floatValue]];
+    CGFloat defaultPlaybackSpeed = [[defaults objectForKey:kVLCSettingPlaybackSpeedDefaultValue] floatValue];
+    if (defaultPlaybackSpeed != 0.)
+        [_mediaPlayer setRate: defaultPlaybackSpeed];
     int deinterlace = [[defaults objectForKey:kVLCSettingDeinterlace] intValue];
     [_mediaPlayer setDeinterlace:deinterlace withFilter:@"blend"];
 
@@ -299,14 +320,19 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
         APLog(@"%s: locking failed", __PRETTY_FUNCTION__);
         return;
     }
+    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
 
     // Set last selected equalizer profile if enabled
-    _mediaPlayer.equalizerEnabled = ![[NSUserDefaults standardUserDefaults] boolForKey:kVLCSettingEqualizerProfileDisabled];
+    _mediaPlayer.equalizerEnabled = ![userDefaults boolForKey:kVLCSettingEqualizerProfileDisabled];
 
     if (_mediaPlayer.equalizerEnabled) {
-        unsigned int profile = (unsigned int)[[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingEqualizerProfile] integerValue];
+        unsigned int profile = (unsigned int)[userDefaults integerForKey:kVLCSettingEqualizerProfile];
         [_mediaPlayer resetEqualizerFromProfile:profile];
+        /* make sure the preset's preamp setting is deployed */
         [_mediaPlayer setPreAmplification:[_mediaPlayer preAmplification]];
+    } else {
+        /* the default is 6 dB leading to audio compression */
+        [_mediaPlayer setPreAmplification:0.];
     }
 
     _mediaWasJustStarted = YES;
@@ -393,7 +419,6 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
     for (NSURL *url in _openedLocalURLs) {
         [url stopAccessingSecurityScopedResource];
-        NSLog(@"%@", url);
     }
     _openedLocalURLs = nil;
     _openedLocalURLs = [[NSMutableArray alloc] init];
@@ -1076,8 +1101,9 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
 - (void)setAmplification:(CGFloat)amplification forBand:(unsigned int)index
 {
-    if (!_mediaPlayer.equalizerEnabled)
+    if (!_mediaPlayer.equalizerEnabled) {
         [_mediaPlayer setEqualizerEnabled:YES];
+    }
 
     [_mediaPlayer setAmplification:amplification forBand:index];
 
@@ -1097,12 +1123,20 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
 - (void)resetEqualizerFromProfile:(unsigned int)profile
 {
-    _mediaPlayer.equalizerEnabled = profile != 0;
-    [[NSUserDefaults standardUserDefaults] setBool:profile == 0 forKey:kVLCSettingEqualizerProfileDisabled];
-    if (profile != 0) {
-        [[NSUserDefaults standardUserDefaults] setObject:@(profile - 1) forKey:kVLCSettingEqualizerProfile];
-        [_mediaPlayer resetEqualizerFromProfile:profile - 1];
+    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
+    if (profile == 0) {
+        _mediaPlayer.equalizerEnabled = NO;
+        [userDefaults setBool:YES forKey:kVLCSettingEqualizerProfileDisabled];
+        [_mediaPlayer setPreAmplification:0.];
+        return;
     }
+
+    [userDefaults setBool:NO forKey:kVLCSettingEqualizerProfileDisabled];
+    _mediaPlayer.equalizerEnabled = YES;
+
+    unsigned int actualProfile = profile - 1;
+    [userDefaults setInteger:actualProfile forKey:kVLCSettingEqualizerProfile];
+    [_mediaPlayer resetEqualizerFromProfile:actualProfile];
 }
 
 - (void)setPreAmplification:(CGFloat)preAmplification
@@ -1128,6 +1162,18 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     return [_mediaPlayer frequencyOfBandAtIndex:index];
 }
 
+- (unsigned int)selectedEqualizerProfile
+{
+    /* this is a bit complex, if the eq is off, we need to return 0
+     * if it is on, we need to provide the profile + 1 as the UI fakes a "Off" profile in its list */
+    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
+    if ([userDefaults boolForKey:kVLCSettingEqualizerProfileDisabled]) {
+        return 0;
+    }
+    unsigned int actualProfile = (unsigned int)[userDefaults integerForKey:kVLCSettingEqualizerProfile];
+    return actualProfile + 1;
+}
+
 #pragma mark - AVAudioSession Notification Observers
 
 - (void)handleInterruption:(NSNotification *)notification
@@ -1204,15 +1250,17 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 {
     if (_needsMetadataUpdate == NO) {
         _needsMetadataUpdate = YES;
-        dispatch_async(dispatch_get_main_queue(), ^{
+        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_LOW, 0), ^{
 #if TARGET_OS_IOS
             VLCMLMedia *media = self->_mediaPlayer.media ? [self->_delegate mediaForPlayingMedia:self->_mediaPlayer.media] : nil;
             [self->_metadata updateMetadataFromMedia:media mediaPlayer:self->_mediaPlayer];
 #else
             [self->_metadata updateMetadataFromMediaPlayer:self->_mediaPlayer];
 #endif
-            self->_needsMetadataUpdate = NO;
-            [self recoverDisplayedMetadata];
+            dispatch_async(dispatch_get_main_queue(), ^{
+                self->_needsMetadataUpdate = NO;
+                [self recoverDisplayedMetadata];
+            });
         });
     }
 }
@@ -1257,6 +1305,46 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     }
     [self restoreAudioAndSubtitleTrack];
 }
+
+- (void)_findCachedSubtitlesForMedia:(VLCMedia *)media
+{
+    /* if we already enforce a subtitle e.g. through Google Drive, don't try to find another */
+    if (_pathToExternalSubtitlesFile) {
+        return;
+    }
+    NSURL *mediaURL = media.url;
+    if (mediaURL.isFileURL) {
+        /* let's see if it is in the Inbox folder or outside our Documents folder and if yes, maybe we have a cached subtitles file? */
+        NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
+        NSString *documentFolderPath = [searchPaths firstObject];
+        NSString *potentialInboxFolderPath = [documentFolderPath stringByAppendingPathComponent:@"Inbox"];
+        NSFileManager *fileManager = [NSFileManager defaultManager];
+        NSString *mediaURLpath = mediaURL.path;
+        if ([mediaURLpath containsString:potentialInboxFolderPath] && ![mediaURLpath containsString:documentFolderPath]) {
+            NSString *mediaFileName = mediaURL.path.lastPathComponent.stringByDeletingPathExtension;
+            searchPaths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
+            NSString *cachefolderPath = [searchPaths.firstObject stringByAppendingPathComponent:kVLCSubtitlesCacheFolderName];
+
+            NSDirectoryEnumerator *folderEnumerator = [fileManager enumeratorAtURL:[NSURL fileURLWithPath:cachefolderPath]
+                                                        includingPropertiesForKeys:@[NSURLNameKey]
+                                                                           options:NSDirectoryEnumerationSkipsSubdirectoryDescendants
+                                                                      errorHandler:nil];
+            NSString *theSubtitleFileName;
+            for (NSURL *theURL in folderEnumerator) {
+                NSString *iter;
+                [theURL getResourceValue:&iter forKey:NSURLNameKey error:NULL];
+
+                if ([iter hasPrefix:mediaFileName]) {
+                    theSubtitleFileName = iter;
+                    break;
+                }
+            }
+
+            NSURL *subtitleURL = [NSURL fileURLWithPath:[cachefolderPath stringByAppendingPathComponent:theSubtitleFileName]];
+            [_mediaPlayer addPlaybackSlave:subtitleURL type:VLCMediaPlaybackSlaveTypeSubtitle enforce:YES];
+        }
+    }
+}
 #endif
 
 - (void)recoverDisplayedMetadata
@@ -1416,7 +1504,6 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
     return @{ kVLCSettingNetworkCaching : [defaults objectForKey:kVLCSettingNetworkCaching],
-              kVLCSettingStretchAudio : [[defaults objectForKey:kVLCSettingStretchAudio] boolValue] ? kVLCSettingStretchAudioOnValue : kVLCSettingStretchAudioOffValue,
               kVLCSettingTextEncoding : [defaults objectForKey:kVLCSettingTextEncoding],
               kVLCSettingSkipLoopFilter : [defaults objectForKey:kVLCSettingSkipLoopFilter],
               kVLCSettingHardwareDecoding : [defaults objectForKey:kVLCSettingHardwareDecoding],
@@ -1453,6 +1540,10 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
 - (void)mediaListPlayer:(VLCMediaListPlayer *)player nextMedia:(VLCMedia *)media
 {
+#if TARGET_OS_IOS
+    [self _findCachedSubtitlesForMedia:media];
+#endif
+
     if ([_delegate respondsToSelector:@selector(playbackService:nextMedia:)]) {
         [_delegate playbackService:self nextMedia:media];
     }
